@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2002-2012 The Cacti Group                                 |
+ | Copyright (C) 2004-2017 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -78,6 +78,7 @@ void set_option(const char *option, const char *value) {
  */
 static const char *getsetting(MYSQL *psql, const char *setting) {
 	char      qstring[256];
+	char      *retval;
 	MYSQL_RES *result;
 	MYSQL_ROW mysql_row;
 	int       i;
@@ -89,7 +90,8 @@ static const char *getsetting(MYSQL *psql, const char *setting) {
 	for (i=0; i<nopts; i++) {
 		if (STRIMATCH(setting, opttable[i].opt)) {
 			/* FOUND IT! */
-			return opttable[i].val;
+			retval = strdup(opttable[i].val);
+			return retval;
 		}
 	}
 
@@ -97,9 +99,21 @@ static const char *getsetting(MYSQL *psql, const char *setting) {
 
 	result = db_query(psql, qstring);
 
-	if ((mysql_num_rows(result) > 0) &&
-		(mysql_row = mysql_fetch_row(result)) != 0)	{
-		return mysql_row[0];
+	if (result != 0) {
+		if (mysql_num_rows(result) > 0) {
+			mysql_row = mysql_fetch_row(result);
+		
+			if (mysql_row != NULL) {
+				retval = strdup(mysql_row[0]);
+				db_free_result(result);
+				return retval;
+			}else{
+				return 0;
+			}
+		}else{
+			db_free_result(result);
+			return 0;
+		}
 	}else{
 		return 0;
 	}
@@ -129,6 +143,7 @@ static int getboolsetting(MYSQL *psql, const char *setting, int dflt) {
 		STRIMATCH(rc, "yes" ) ||
 		STRIMATCH(rc, "true") ||
 		STRIMATCH(rc, "1"   ) ) {
+		free((char *)rc);
 		return TRUE;
 	}
 
@@ -136,10 +151,12 @@ static int getboolsetting(MYSQL *psql, const char *setting, int dflt) {
 		STRIMATCH(rc, "no"   ) ||
 		STRIMATCH(rc, "false") ||
 		STRIMATCH(rc, "0"    ) ) {
+		free((char *)rc);
 		return FALSE;
 	}
 
 	/* doesn't really match one of our keywords: what to do? */
+	free((char *)rc);
 
 	return dflt;
 }
@@ -156,6 +173,7 @@ static int getboolsetting(MYSQL *psql, const char *setting, int dflt) {
  */
 static const char *getglobalvariable(MYSQL *psql, const char *setting) {
 	char      qstring[256];
+	char      *retval;
 	MYSQL_RES *result;
 	MYSQL_ROW mysql_row;
 	int       i;
@@ -175,14 +193,46 @@ static const char *getglobalvariable(MYSQL *psql, const char *setting) {
 
 	result = db_query(psql, qstring);
 
-	if ((mysql_num_rows(result) > 0) &&
-		(mysql_row = mysql_fetch_row(result)) != 0)	{
-		return mysql_row[1];
+	if (result != 0) {
+		if (mysql_num_rows(result) > 0) {
+			mysql_row = mysql_fetch_row(result);
+		
+			if (mysql_row != NULL) {
+				retval = strdup(mysql_row[1]);
+				db_free_result(result);
+				return retval;
+			}else{
+				return 0;
+			}
+		}else{
+			db_free_result(result);
+			return 0;
+		}
 	}else{
 		return 0;
 	}
 }
 
+/*! \fn int is_debug_device(int device_id)
+ *  \brief Determine if a device is a debug device
+ *
+ */
+int is_debug_device(int device_id) {
+	extern int *debug_devices;
+	int i = 0;
+
+	while(i < 100) {
+		if (debug_devices[i] == '\0') break;
+		if (debug_devices[i] == device_id) {
+			return TRUE;
+		}
+
+		i++;
+	}
+
+	return FALSE;
+}
+
 /*! \fn void read_config_options(void)
  *  \brief Reads the default Spine runtime parameters from the database and set's the global array
  *
@@ -201,24 +251,27 @@ void read_config_options() {
 
 	/* get the mysql server version */
 	set.dbversion = 0;
-	if ((res = getglobalvariable(&mysql, "version")) != 0 ) {
+	if ((res = getglobalvariable(&mysql, "version")) != 0) {
 		set.dbversion = atoi(res);
+		free((char *)res);
 	}
 
 	/* get logging level from database - overrides spine.conf */
-	if ((res = getsetting(&mysql, "log_verbosity")) != 0 ) {
+	if ((res = getsetting(&mysql, "log_verbosity")) != 0) {
 		const int n = atoi(res);
+		free((char *)res);
 		if (n != 0) set.log_level = n;
 	}
 
 	/* determine script server path operation and default log file processing */
-	if ((res = getsetting(&mysql, "path_webroot")) != 0 ) {
+	if ((res = getsetting(&mysql, "path_webroot")) != 0) {
 		snprintf(set.path_php_server, SMALL_BUFSIZE, "%s/script_server.php", res);
 		snprintf(web_root, BUFSIZE, "%s", res);
+		free((char *)res);
 	}
 
 	/* determine logfile path */
-	if ((res = getsetting(&mysql, "path_cactilog")) != 0 ) {
+	if ((res = getsetting(&mysql, "path_cactilog")) != 0) {
 		if (strlen(res) != 0) {
 			snprintf(set.path_logfile, SMALL_BUFSIZE, "%s", res);
 		}else{
@@ -228,6 +281,7 @@ void read_config_options() {
 				set.path_logfile[0] ='\0';
 			}
 		}
+		free((char *)res);
 	}else{
 		snprintf(set.path_logfile, SMALL_BUFSIZE, "%s/log/cacti.log", web_root);
  	}
@@ -239,8 +293,9 @@ void read_config_options() {
 	SPINE_LOG_DEBUG(("DEBUG: The path_cactilog variable is %s", set.path_logfile));
 
 	/* determine log file, syslog or both, default is 1 or log file only */
-	if ((res = getsetting(&mysql, "log_destination")) != 0 ) {
+	if ((res = getsetting(&mysql, "log_destination")) != 0) {
 		set.log_destination = parse_logdest(res, LOGDEST_FILE);
+		free((char *)res);
 	}else{
 		set.log_destination = LOGDEST_FILE;
 	}
@@ -252,24 +307,27 @@ void read_config_options() {
 	set.logfile_processed = TRUE;
 
 	/* get PHP Path Information for Scripting */
-	if ((res = getsetting(&mysql, "path_php_binary")) != 0 ) {
+	if ((res = getsetting(&mysql, "path_php_binary")) != 0) {
 		STRNCOPY(set.path_php, res);
+		free((char *)res);
 	}
 
 	/* log the path_php variable */
 	SPINE_LOG_DEBUG(("DEBUG: The path_php variable is %s", set.path_php));
 
 	/* set availability_method */
-	if ((res = getsetting(&mysql, "availability_method")) != 0 ) {
+	if ((res = getsetting(&mysql, "availability_method")) != 0) {
 		set.availability_method = atoi(res);
+		free((char *)res);
 	}
 
 	/* log the availability_method variable */
 	SPINE_LOG_DEBUG(("DEBUG: The availability_method variable is %i", set.availability_method));
 
 	/* set ping_recovery_count */
-	if ((res = getsetting(&mysql, "ping_recovery_count")) != 0 ) {
+	if ((res = getsetting(&mysql, "ping_recovery_count")) != 0) {
 		set.ping_recovery_count = atoi(res);
+		free((char *)res);
 	}
 
 	/* log the ping_recovery_count variable */
@@ -278,38 +336,43 @@ void read_config_options() {
 	/* set ping_failure_count */
 	if ((res = getsetting(&mysql, "ping_failure_count")) != 0) {
 		set.ping_failure_count = atoi(res);
+		free((char *)res);
 	}
 
 	/* log the ping_failure_count variable */
 	SPINE_LOG_DEBUG(("DEBUG: The ping_failure_count variable is %i", set.ping_failure_count));
 
 	/* set ping_method */
-	if ((res = getsetting(&mysql, "ping_method")) != 0 ) {
+	if ((res = getsetting(&mysql, "ping_method")) != 0) {
 		set.ping_method = atoi(res);
+		free((char *)res);
 	}
 
 	/* log the ping_method variable */
 	SPINE_LOG_DEBUG(("DEBUG: The ping_method variable is %i", set.ping_method));
 
 	/* set ping_retries */
-	if ((res = getsetting(&mysql, "ping_retries")) != 0 ) {
+	if ((res = getsetting(&mysql, "ping_retries")) != 0) {
 		set.ping_retries = atoi(res);
+		free((char *)res);
 	}
 
 	/* log the ping_retries variable */
 	SPINE_LOG_DEBUG(("DEBUG: The ping_retries variable is %i", set.ping_retries));
 
 	/* set ping_timeout */
-	if ( (res = getsetting(&mysql, "ping_timeout")) != 0 ) {
+	if ( (res = getsetting(&mysql, "ping_timeout")) != 0) {
 		set.ping_timeout = atoi(res);
+		free((char *)res);
 	}
 
 	/* log the ping_timeout variable */
 	SPINE_LOG_DEBUG(("DEBUG: The ping_timeout variable is %i", set.ping_timeout));
 
 	/* set snmp_retries */
-	if ( (res = getsetting(&mysql, "snmp_retries")) != 0 ) {
+	if ( (res = getsetting(&mysql, "snmp_retries")) != 0) {
 		set.snmp_retries = atoi(res);
+		free((char *)res);
 	}
 
 	/* log the snmp_retries variable */
@@ -340,8 +403,9 @@ void read_config_options() {
 	SPINE_LOG_DEBUG(("DEBUG: The log_pstats variable is %i", set.log_pstats));
 
 	/* get Cacti defined max threads override spine.conf */
-	if ((res = getsetting(&mysql, "max_threads")) != 0 ) {
+	if ((res = getsetting(&mysql, "max_threads")) != 0) {
 		set.threads = atoi(res);
+		free((char *)res);
 		if (set.threads > MAX_THREADS) {
 			set.threads = MAX_THREADS;
 		}
@@ -351,8 +415,9 @@ void read_config_options() {
 	SPINE_LOG_DEBUG(("DEBUG: The threads variable is %i", set.threads));
 
 	/* get the poller_interval for those who have elected to go with a 1 minute polling interval */
-	if ((res = getsetting(&mysql, "poller_interval")) != 0 ) {
+	if ((res = getsetting(&mysql, "poller_interval")) != 0) {
 		set.poller_interval = atoi(res);
+		free((char *)res);
 	}else{
 		set.poller_interval = 0;
 	}
@@ -365,8 +430,9 @@ void read_config_options() {
 	}
 
 	/* get the concurrent_processes variable to determine thread sleep values */
-	if ((res = getsetting(&mysql, "concurrent_processes")) != 0 ) {
+	if ((res = getsetting(&mysql, "concurrent_processes")) != 0) {
 		set.num_parent_processes = atoi(res);
+		free((char *)res);
 	}else{
 		set.num_parent_processes = 1;
 	}
@@ -375,8 +441,9 @@ void read_config_options() {
 	SPINE_LOG_DEBUG(("DEBUG: The number of concurrent processes is %i", set.num_parent_processes));
 
 	/* get the script timeout to establish timeouts */
-	if ((res = getsetting(&mysql, "script_timeout")) != 0 ) {
+	if ((res = getsetting(&mysql, "script_timeout")) != 0) {
 		set.script_timeout = atoi(res);
+		free((char *)res);
 		if (set.script_timeout < 5) {
 			set.script_timeout = 5;
 		}
@@ -387,9 +454,28 @@ void read_config_options() {
 	/* log the script timeout value */
 	SPINE_LOG_DEBUG(("DEBUG: The script timeout is %i", set.script_timeout));
 
+	/* get selective_device_debug string */
+	if ((res = getsetting(&mysql, "selective_device_debug")) != 0) {
+		STRNCOPY(set.selective_device_debug, res);
+		free((char *)res);
+	}
+
+	/* log the selective_device_debug variable */
+	SPINE_LOG_DEBUG(("DEBUG: The selective_device_debug variable is %s", set.selective_device_debug));
+
+	/* get spine_log_level */
+	if ((res = getsetting(&mysql, "spine_log_level")) != 0) {
+		set.spine_log_level = atoi(res);
+		free((char *)res);
+	}
+
+	/* log the spine_log_level variable */
+	SPINE_LOG_DEBUG(("DEBUG: The spine_log_level variable is %i", set.spine_log_level));
+
 	/* get the number of script server processes to run */
-	if ((res = getsetting(&mysql, "php_servers")) != 0 ) {
+	if ((res = getsetting(&mysql, "php_servers")) != 0) {
 		set.php_servers = atoi(res);
+		free((char *)res);
 
 		if (set.php_servers > MAX_PHP_SERVERS) {
 			set.php_servers = MAX_PHP_SERVERS;
@@ -427,10 +513,11 @@ void read_config_options() {
 
 		result = db_query(&mysql, sqlbuf);
 		num_rows = mysql_num_rows(result);
+		db_free_result(result);
 
 		if (num_rows > 0) set.php_required = TRUE;
 
-		SPINE_LOG_DEBUG(("DEBUG: StartHost='%i', EndHost='%i', TotalPHPScripts='%i'",
+		SPINE_LOG_DEBUG(("DEBUG: StartDevice='%i', EndDevice='%i', TotalPHPScripts='%i'",
 			set.start_host_id,
 			set.end_host_id,
 			num_rows));
@@ -446,10 +533,11 @@ void read_config_options() {
 
 		result = db_query(&mysql, sqlbuf);
 		num_rows = mysql_num_rows(result);
+		db_free_result(result);
 
 		if (num_rows > 0) set.php_required = TRUE;
 
-		SPINE_LOG_DEBUG(("DEBUG: Host List to be polled='%s', TotalPHPScripts='%i'",
+		SPINE_LOG_DEBUG(("DEBUG: Device List to be polled='%s', TotalPHPScripts='%i'",
 			set.host_id_list,
 			num_rows));
 	}
@@ -460,8 +548,9 @@ void read_config_options() {
 		: "Not "));
 
 	/* determine the maximum oid's to obtain in a single get request */
-	if ((res = getsetting(&mysql, "max_get_size")) != 0 ) {
+	if ((res = getsetting(&mysql, "max_get_size")) != 0) {
 		set.snmp_max_get_size = atoi(res);
+		free((char *)res);
 
 		if (set.snmp_max_get_size > 128) {
 			set.snmp_max_get_size = 128;
@@ -473,7 +562,6 @@ void read_config_options() {
 	/* log the snmp_max_get_size variable */
 	SPINE_LOG_DEBUG(("DEBUG: The Maximum SNMP OID Get Size is %i", set.snmp_max_get_size));
 
-	mysql_free_result(result);
 	db_disconnect(&mysql);
 }
 
@@ -507,13 +595,22 @@ int read_spine_config(char *file) {
 			if (!feof(fp) && *buff != '#' && *buff != ' ' && *buff != '\n') {
 				sscanf(buff, "%15s %255s", p1, p2);
 
-				if (STRIMATCH(p1, "DB_Host"))               STRNCOPY(set.dbhost, p2);
+				if (STRIMATCH(p1, "RDB_Host"))              STRNCOPY(set.rdbhost, p2);
+				else if (STRIMATCH(p1, "RDB_Database"))     STRNCOPY(set.rdbdb, p2);
+				else if (STRIMATCH(p1, "RDB_User"))         STRNCOPY(set.rdbuser, p2);
+				else if (STRIMATCH(p1, "RDB_Pass"))         STRNCOPY(set.rdbpass, p2);
+				else if (STRIMATCH(p1, "RDB_Port"))         set.rdbport    = atoi(p2);
+				else if (STRIMATCH(p1, "DB_Host"))          STRNCOPY(set.dbhost, p2);
 				else if (STRIMATCH(p1, "DB_Database"))      STRNCOPY(set.dbdb, p2);
 				else if (STRIMATCH(p1, "DB_User"))          STRNCOPY(set.dbuser, p2);
 				else if (STRIMATCH(p1, "DB_Pass"))          STRNCOPY(set.dbpass, p2);
 				else if (STRIMATCH(p1, "DB_Port"))          set.dbport    = atoi(p2);
 				else if (STRIMATCH(p1, "Poller"))           set.poller_id = atoi(p2);
-				else if (STRIMATCH(p1, "DB_PreG"))          set.pre087g   = atoi(p2);
+				else if (STRIMATCH(p1, "DB_PreG")) {
+					if (!set.stderr_notty) {
+						fprintf(stderr,"WARNING: DB_PreG is no longer supported\n");
+					}
+				}
 				else if (STRIMATCH(p1, "SNMP_Clientaddr"))  STRNCOPY(set.snmp_clientaddr, p2);
 				else if (!set.stderr_notty) {
 					fprintf(stderr,"WARNING: Unrecongized directive: %s=%s in %s\n", p1, p2, file);
@@ -537,14 +634,23 @@ int read_spine_config(char *file) {
  */
 void config_defaults() {
 	set.threads = DEFAULT_THREADS;
+
+	/* default server */
 	set.dbport  = DEFAULT_DB_PORT;
-	set.pre087g = DEFAULT_DB_PREG;
 
 	STRNCOPY(set.dbhost, DEFAULT_DB_HOST);
 	STRNCOPY(set.dbdb,   DEFAULT_DB_DB  );
 	STRNCOPY(set.dbuser, DEFAULT_DB_USER);
 	STRNCOPY(set.dbpass, DEFAULT_DB_PASS);
 
+	/* remote default server */
+	set.rdbport  = DEFAULT_DB_PORT;
+
+	STRNCOPY(set.rdbhost, DEFAULT_DB_HOST);
+	STRNCOPY(set.rdbdb,   DEFAULT_DB_DB  );
+	STRNCOPY(set.rdbuser, DEFAULT_DB_USER);
+	STRNCOPY(set.rdbpass, DEFAULT_DB_PASS);
+
 	STRNCOPY(config_paths[0], CONFIG_PATH_1);
 	STRNCOPY(config_paths[1], CONFIG_PATH_2);
 	STRNCOPY(config_paths[2], CONFIG_PATH_3);
@@ -564,11 +670,18 @@ void die(const char *format, ...) {
 	va_list	args;
 	char logmessage[BUFSIZE];
 	char flogmessage[BUFSIZE];
+	int old_errno = errno;
 
 	va_start(args, format);
 	vsprintf(logmessage, format, args);
 	va_end(args);
 
+	if (set.log_perror) {
+		char perr[BUFSIZE];
+		snprintf(perr, BUFSIZE, " [%d, %s]", old_errno, strerror(old_errno));
+		strcat(logmessage,perr);
+	}
+
 	if (set.logfile_processed) {
 		if (set.parent_fork == SPINE_PARENT) {
 			snprintf(flogmessage, BUFSIZE, "%s (Spine parent)", logmessage);
@@ -614,7 +727,7 @@ int spine_log(const char *format, ...) {
 	char flogmessage[LOGSIZE];     /* Formatted Log Message */
 
 	va_start(args, format);
-	vsprintf(ulogmessage, format, args);
+	vsnprintf(ulogmessage, LOGSIZE - 1, format, args);
 	va_end(args);
 
 	/* default for "console" messages to go to stdout */
@@ -639,7 +752,7 @@ int spine_log(const char *format, ...) {
 	localtime_r(&nowbin,&now_time);
 	now_ptr = &now_time;
 
-	if (strftime(flogmessage, 50, "%m/%d/%Y %I:%M:%S %p - ", now_ptr) == (size_t) 0) {
+	if (strftime(flogmessage, 50, "%Y-%m-%d %H:%M:%S - ", now_ptr) == (size_t) 0) {
 		#ifdef DISABLE_STDERR
 		fp = stdout;
 		#else
@@ -655,8 +768,8 @@ int spine_log(const char *format, ...) {
 		}
 	}
 
-	strncat(flogmessage, logprefix,   sizeof(flogmessage)-1);
-	strncat(flogmessage, ulogmessage, sizeof(flogmessage)-1);
+	strncat(flogmessage, logprefix,   sizeof(flogmessage) - 1);
+	strncat(flogmessage, ulogmessage, sizeof(flogmessage) - 50);
 
 	if ((IS_LOGGING_TO_FILE() &&
 		(set.log_level != POLLER_VERBOSITY_NONE) &&
@@ -794,14 +907,14 @@ int is_ipaddress(const char *string) {
  *  \return TRUE if long or double, FALSE if not
  *
  */
-int is_numeric(const char *string) {
+int is_numeric(char *string) {
 	long local_lval;
 	double local_dval;
 	char *end_ptr_long, *end_ptr_double;
 	int conv_base=10;
 	int length;
 
-	length = strlen(string);
+	length = strlen(trim(string));
 
 	if (!length) {
 		return FALSE;
@@ -810,6 +923,7 @@ int is_numeric(const char *string) {
  	/* check for an integer */
 	errno = 0;
 	local_lval = strtol(string, &end_ptr_long, conv_base);
+
 	if (errno != ERANGE) {
 		if (end_ptr_long == string + length) { /* integer string */
 			return TRUE;
@@ -836,15 +950,7 @@ int is_numeric(const char *string) {
 		end_ptr_double = NULL;
 	}
 
-	if (!errno) {
-		if (STRIMATCH(string," ")) {
-			return FALSE;
-		}else{
-			return TRUE;
-		}
-	}else{
-		return FALSE;
-	}
+	return FALSE;
 }
 
 /*! \fn int is_hexadecimal(const char *str, const short ignore_space)
@@ -1221,24 +1327,33 @@ int hasCaps() {
 	cap_value_t capval;
 	cap_flag_value_t capflag;
 
-	caps = cap_from_text("cap_net_raw=eip");
+	/* Recommended caps: cap_net_raw=eip */
+	caps = cap_get_proc();
 	if (caps == NULL) {
-		SPINE_LOG_DEBUG(("WARNING: cap_from_text failed."));
+		SPINE_LOG(("ERROR: cap_get_proc failed."));
 		return FALSE;
 	}
 
-	for (capval=0; ;capval++) {
-		if (cap_get_flag(caps, capval, CAP_EFFECTIVE, &capflag))
-			break;
-		if (capflag != CAP_SET)
-			return FALSE;
+    /* check if cap_net_raw is in effective set */
+	if (cap_get_flag(caps, CAP_NET_RAW, CAP_EFFECTIVE, &capflag)) {
+		SPINE_LOG(("ERROR: cap_get_flag for CAP_NET_RAW failed. ICMP ping will not work as non-root user."));
+		return FALSE;
 	}
+
+	if (capflag != CAP_SET) {
+		SPINE_LOG(("ERROR: Capability CAP_NET_RAW is not set. ICMP ping will not work as non-root user."));
+		return FALSE;
+	}
+
+	SPINE_LOG_DEBUG(("DEBUG: Capability CAP_NET_RAW is set."));
+	cap_free(caps);
+
 	return TRUE;
 	#else
 	return FALSE;
 	#endif
 }
-	
+
 void checkAsRoot() {
 	#ifndef __CYGWIN__
 	#ifdef SOLAR_PRIV
@@ -1255,7 +1370,7 @@ void checkAsRoot() {
 	}
 
 	/* Add priviledge to send/receive ICMP packets */
-	if (priv_addset(privset, PRIV_NET_ICMPACCESS) < 0 ) {
+	if (priv_addset(privset, PRIV_NET_ICMPACCESS) < 0) {
 		SPINE_LOG_DEBUG(("Warning: Addition of PRIV_NET_ICMPACCESS to privset failed: '%s'.", strerror(errno)));
 	}
 
@@ -1286,7 +1401,7 @@ void checkAsRoot() {
 		seteuid(0);
 	
 		if (geteuid() != 0) {
-			SPINE_LOG_DEBUG(("WARNING: Spine NOT running asroot.  This is required if using ICMP.  Please run \"chmod +s;chown root:root spine\" to resolve."));
+			SPINE_LOG_DEBUG(("WARNING: Spine NOT running asroot.  This is required if using ICMP.  Please run \"chown root:root spine;chmod +s spine\" to resolve."));
 			set.icmp_avail = FALSE;
 		}else{
 			SPINE_LOG_DEBUG(("DEBUG: Spine is running asroot."));
